<?php
/**
 * Plugin Name: Protect Admin Account
 * Plugin URI: https://10horizonsplugins.com/wordpress-plugins/
 * Description: Use this plugin to protect admin accounts from being deleted or modified by other users.
 * Version: 0.2.1
 * Author: 10Horizons Plugins
 * Author URI: https://10horizonsplugins.com
 * 
 */

if ( ! defined( 'ABSPATH' ) ) exit;

define ( 'thp-paa-protect-admin-plugin', TRUE );
include ( 'paa-options.php' );

if (!function_exists('thp_paa_save_activator_id')) {
	
	function thp_paa_save_activator_id() {
		if ( (current_user_can( 'activate_plugins' )) && (current_user_can( 'manage_options' )) ) {
			$current_user = wp_get_current_user();
			$activator_id = filter_var($current_user->ID, FILTER_VALIDATE_INT);
			
			$thp_paa_options = get_option( 'thp_paa_options' );
			$thp_paa_options['activator_id'] = $activator_id;
			
			update_option( 'thp_paa_options', $thp_paa_options );
		}
	}
	register_activation_hook( __FILE__, 'thp_paa_save_activator_id' );

}

if (!function_exists('thp_paa_get_activator_id')) {
		
	function thp_paa_get_activator_id() {
		$thp_paa_options = get_option( 'thp_paa_options' );
		$activator_id = ( !empty($thp_paa_options['activator_id']) ? $thp_paa_options['activator_id'] : '' );
		return $activator_id;
	}
}

if (!function_exists('thp_paa_get_protected_user_id')) {
	
	function thp_paa_get_protected_user_id() {
		
		$thp_paa_options = get_option( 'thp_paa_options' );
		
		$old_protected_user_id = ( !empty($thp_paa_options['protect_user1']) ? $thp_paa_options['protect_user1'] : '' ); //backward compatibility v0.1.0
		$old_protected_user_id = (int) $old_protected_user_id;
		
		$protected_user_ids = ( !empty( $thp_paa_options['protected_users'] ) ? $thp_paa_options['protected_users'] : '' );
		
		$protected_user_ids = ( is_array($protected_user_ids) ? array_keys($protected_user_ids) : $protected_user_ids );
		
		if ($old_protected_user_id)
			$protected_user_ids[] = $old_protected_user_id;
		
		$protected_user_ids = ( is_array($protected_user_ids) ? array_unique( $protected_user_ids ) : $protected_user_ids );
		
		return $protected_user_ids;
	}
}

if (!function_exists('thp_paa_protection_is_enable')) {
	
	function thp_paa_protection_is_enable() {
		$thp_paa_options = get_option( 'thp_paa_options' );
		$enable = ( !empty($thp_paa_options['enable']) ? $thp_paa_options['enable'] : '' );
		
		if ($enable)
			return true;
		else
			return false;
	}
}

if (!function_exists('thp_paa_user_is_activator')) {
	
	function thp_paa_user_is_activator() {
		$current_user = wp_get_current_user()->ID;
		$activator_id = thp_paa_get_activator_id();
		
		if ($current_user == $activator_id)
			return true;
		else if ($current_user != $activator_id)
			return false;
	}
}

if (!function_exists('thp_paa_prevent_user_deletion')) {

	function thp_paa_prevent_user_deletion( $userid ) {
		
		if ( !thp_paa_protection_is_enable() )
			return;
		
		$admins = thp_paa_get_protected_user_id();
		
		if ($admins && is_array($admins)) {
			foreach ($admins as $admin) {
				if ( $userid == $admin ) {
					if ( thp_paa_user_is_activator() ) {
						return;
					}
					elseif ( !thp_paa_user_is_activator() ) {
						$user_obj = get_user_by('id', $userid);
						$name = $user_obj->user_login;
						
						wp_die( sprintf( __( 'Sorry, you are not allowed to delete user %s. If you are deleting users in bulk, some users before this might have been successfully deleted.', 'protect-admin-account' ), $name ) );
					}
				}
			}
		}
	}
	
	add_action('delete_user', 'thp_paa_prevent_user_deletion');
}

if (!function_exists('thp_paa_prevent_edit_user_url_access')) {
	
	function thp_paa_prevent_edit_user_url_access() {
		global $pagenow;
		
		if ( $pagenow == 'user-edit.php' ) {
			
			if ( !thp_paa_protection_is_enable() )
				return;
			
			$userid = filter_input(INPUT_GET, 'user_id', FILTER_VALIDATE_INT);
			$admins = thp_paa_get_protected_user_id();
			
			if ($admins && is_array($admins)) {
				foreach ($admins as $admin) {
					if ( $userid == $admin ) {
						if ( thp_paa_user_is_activator() ) {
							return;
						}
						elseif ( !thp_paa_user_is_activator() ) {
							wp_die( __( 'Sorry, you are not allowed to edit this user.', 'protect-admin-account' ) );
						}
					}
				}
			}
		}
	}
	
	add_action('admin_init', 'thp_paa_prevent_edit_user_url_access');
}

if (!function_exists('thp_paa_prevent_update_user')) {
	
	function thp_paa_prevent_update_user($userid) {
		
		if ( !thp_paa_protection_is_enable() )
			return;
		
		$admins = thp_paa_get_protected_user_id();
			
		if ($admins && is_array($admins)) {
			foreach ($admins as $admin) {
				if ( $userid == $admin ) {
					if ( thp_paa_user_is_activator() ) {
						return;
					}
					elseif ( !thp_paa_user_is_activator() ) {
						wp_die( __( 'Sorry, you are not allowed to edit this user.', 'protect-admin-account' ) );
					}
				}
			}
		}
	}
	
	add_action('edit_user_profile_update', 'thp_paa_prevent_update_user');
}

if (!function_exists('thp_paa_hide_pwd_fields')) {
	
	function thp_paa_hide_pwd_fields() {
		
		if (isset($_GET['user_id'])) {
			
			if ( !thp_paa_protection_is_enable() )
				return true;
			
			$userid = filter_input(INPUT_GET, 'user_id', FILTER_VALIDATE_INT);
			$admins = thp_paa_get_protected_user_id();
			
			if ($admins && is_array($admins)) {
				foreach ($admins as $admin) {
					if ( $userid == $admin ) {
						if ( thp_paa_user_is_activator() )
							return true;
						elseif ( !thp_paa_user_is_activator() )
							return false;
					}
				}
			}
		}
	return true;
	}
	
	add_filter('show_password_fields', 'thp_paa_hide_pwd_fields');
}

if (!function_exists('thp_paa_remove_checkbox')) {
	
	function thp_paa_remove_checkbox() {
		global $pagenow;
		
		if ( $pagenow == 'users.php' ) {
			
			if ( !thp_paa_protection_is_enable() )
				return;
			
			$current_user = wp_get_current_user()->ID;
			$admins = thp_paa_get_protected_user_id();
			
			if ($admins && is_array($admins)) {
				
				echo '<style type="text/css">';
				
				foreach ($admins as $admin) {
					if ( !thp_paa_user_is_activator() && $admin != $current_user) {
						echo '
						table.users tr#user-'.$admin.' th.check-column input[type="checkbox"] {
							display: none !important;
						}
						';
					}
				}
				
				echo '</style>';
			}
		}
	}
	
	add_action('admin_head', 'thp_paa_remove_checkbox');
}

if (!function_exists('thp_paa_revert_user_role')) {
	
	function thp_paa_revert_user_role($userid, $role, $old_roles) {
		
		if ( !thp_paa_protection_is_enable() )
			return;
		
		$admins = thp_paa_get_protected_user_id();
			
		if ($admins && is_array($admins)) {
			foreach ($admins as $admin) {
				if ( $userid == $admin ) {
					if ( !thp_paa_user_is_activator() ) {
						if (current_user_can('edit_users')) { //security check
							
							$user_object = new WP_User($userid);
							$user_object->remove_role($role);
							
							foreach ($old_roles as $old_role) {
								$user_object->add_role($old_role);
							}
							
							$name = $user_object->user_login;
							
							wp_die( sprintf( __( 'Sorry, you are not allowed to change role of user %s.', 'protect-admin-account' ), $name ) );
						}
					}
				}
			}
		}
	}
	
	add_action('set_user_role', 'thp_paa_revert_user_role', 10, 3);
}

if (!function_exists('thp_paa_revert_email_change')) {
	
	function thp_paa_revert_email_change( $errors, $update, $user_obj ) {
		
		if ( !thp_paa_protection_is_enable() )
			return;
		
		$admins = thp_paa_get_protected_user_id();
			
		if ($admins && is_array($admins)) {
			foreach ($admins as $admin) {
				if ( $user_obj->ID == $admin ) {
					if ( !thp_paa_user_is_activator() ) {
						$old = get_user_by('id', $user_obj->ID);
						
						if ( $user_obj->user_email != $old->user_email ) {
							if ( current_user_can('edit_users') ) { //security check
								$user_obj->user_email = $old->user_email;
								wp_die( __( "Sorry, you are not allowed to change this user's email.", "protect-admin-account" ) );
							}
						}
					}
				}
			}
		}
	}
	
	add_action('user_profile_update_errors', 'thp_paa_revert_email_change', 10, 3);
}

if (!function_exists('thp_paa_hide_plugin')) {
	
	function thp_paa_hide_plugin($plugins) {
		
		//if ( !thp_paa_protection_is_enable() )
		//	return $plugins;
		
		if ( thp_paa_user_is_activator() )
			return $plugins;
		
		unset( $plugins['protect-admin-account/index.php'] );
		
		return $plugins;
	}
	
	add_filter('all_plugins', 'thp_paa_hide_plugin');
}

if (!function_exists('thp_paa_deactivate_plugin')) {
	
	function thp_paa_deactivate_plugin() {
		
		$thp_paa_options = get_option( 'thp_paa_options' );
		
		if ( !$thp_paa_options )
			deactivate_plugins( plugin_basename( __FILE__ ) );
	}
	
	add_action( 'pre_current_active_plugins', 'thp_paa_deactivate_plugin' );
}
