<?php
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! defined( 'thp-paa-protect-admin-plugin' ) ) {
   die('Undefined constant.');
}


if (!function_exists('thp_paa_admin_menu')) {
	function thp_paa_admin_menu() {
		if ( thp_paa_user_is_activator() )
			add_submenu_page( 'users.php', __( 'Protect Admin Account Settings', 'protect-admin-account' ), __( 'Protect User', 'protect-admin-account' ), 'activate_plugins', 'thp-paa-admin-settings', 'thp_paa_admin_settings' );
	}
	add_action('admin_menu', 'thp_paa_admin_menu', 99);
}

if (!function_exists('thp_paa_admin_settings')) {
	
	function thp_paa_admin_settings() {
	
		if (isset($_POST['action'])){
			if ( $_POST['action'] === 'update' )
				thp_paa_options_handler();
		}
		?>
		
		<div class="wrap">
		
			<h1><?php _e( "Protect Admin Account", 'protect-admin-account' ); ?></h1>
			
			<form name="thp_paa_admin_settings_form" method="post" action="">
				
				<?php wp_nonce_field('update-options', 'thp_paa_admin_settings_nonce'); 
				
				$thp_paa_options = get_option( 'thp_paa_options' );
				?>
				
				<table class="form-table" role="presentation">
					<tbody>
					
					<tr>
					<th scope="row"><label for="thp_paa_options[enable]"><?php _e( "Activation", 'protect-admin-account' ); ?> :</label></th>
						<td>
							<input type="checkbox" name="thp_paa_options[enable]" value="1" <?php checked( '1', (!empty($thp_paa_options['enable']) ? $thp_paa_options['enable'] : '') ); ?>>
							<?php _e( "Enable protection?", 'protect-admin-account' ); ?>
						</td>
					</tr>
					
					<tr>
					<th scope="row"><label for="thp_paa_options[protected_users]"><?php _e( "Admin Accounts", 'protect-admin-account' ); ?> :</label></th>
						<td>
						
						<?php
						$old_user_ID = !empty($thp_paa_options['protect_user1']) ? $thp_paa_options['protect_user1'] : 0; //user ID
						$admins = get_users( [ 'role__in' => [ 'administrator' ] ] ); //user objects
						$userchecked = 0;
						
						foreach ( $admins as $admin ) {
							
							if ( ($old_user_ID != 0) && ($admin->ID == $old_user_ID) ) {
								$userchecked = 1;
							}
							else {
								$userchecked = (!empty($thp_paa_options['protected_users'][$admin->ID]) ? $thp_paa_options['protected_users'][$admin->ID] : '');
							}
							
							?>
							
							<fieldset>
								<label for="thp_paa_options[protected_users][<?php echo $admin->ID; ?>]">
									<input type="checkbox" id="thp_paa_options[protected_users][<?php echo $admin->ID; ?>]" name="thp_paa_options[protected_users][<?php echo $admin->ID; ?>]" value="1" <?php checked( '1', $userchecked ); ?>>
									<?php echo $admin->user_login.' ('.$admin->user_email.')'; ?>
								</label>
							</fieldset>
							
						<?php
						} //end of for loop ?>
						
						<br />
						<p class="description"><?php _e( "Tick the user account(s) you want to protect. Only administrator accounts can be protected.", 'protect-admin-account' ); ?></p>
						</td>
					</tr>
					
					</tbody>
				</table>
				
				<p>
					<input name="thp_paa_options[activator_id]" type="hidden" value="<?php echo (!empty($thp_paa_options['activator_id']) ? $thp_paa_options['activator_id'] : ''); ?>">
					<input class="button-primary" type="submit" name="Submit" value="<?php _e( "Save", 'protect-admin-account' ); ?>" />
				</p>
				
				<input type="hidden" name="action" value="update" />
			</form>
			
		</div>
		
	<?php
	}
}

if (!function_exists('thp_paa_options_handler')) {
	
	function thp_paa_options_handler() {
		
		if ( (isset( $_POST['thp_paa_admin_settings_nonce'])) && (wp_verify_nonce( $_POST['thp_paa_admin_settings_nonce'], 'update-options' )) ) {
			if ( (current_user_can( 'activate_plugins' )) && (current_user_can( 'manage_options' )) ) {
				
				$current_user = wp_get_current_user()->ID;
				$activator_id = thp_paa_get_activator_id();
				if ($current_user != $activator_id)
					wp_die( __( 'Sorry, you are not allowed to edit the settings.', 'protect-admin-account' ) );
				
				$new_options = filter_input(INPUT_POST, 'thp_paa_options', FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
				$user_ids = ( !empty( $new_options['protected_users'] ) ? $new_options['protected_users'] : '' );
				$user_ids = ( is_array($user_ids) ?	array_keys($user_ids) : $user_ids );
				
				if ($user_ids) {
					foreach ($user_ids as $user_id) {
						$user_obj = get_user_by('id', $user_id);
						$user_roles = !empty($user_obj->roles) ? $user_obj->roles : array();
						
						if (! in_array( 'administrator', $user_roles, true ) ) {
							echo '<div class="error"><p>';
							_e( "One or more user selected is not admin. All users selected must be admin. Settings are not saved.", 'protect-admin-account' );
							echo '</p></div>';
							return;
						}
					}
				}
				
				$status = update_option( 'thp_paa_options', filter_var($new_options, FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY) );
				
				if ($status) {
					if ( get_option( 'thp_paa_options[protect_user1]' ) ) { //for users of version 0.1.0
						$deletion_status = delete_option( 'thp_paa_options[protect_user1]' );
						
						if ($deletion_status) {
							echo '<div class="updated"><p>';
							_e( "Settings have been updated successfully!", 'protect-admin-account' );
							echo '</p></div>';
						}
						else {
							echo '<div class="error"><p>';
							_e( "Oops, something went wrong. Try saving your settings again.", 'protect-admin-account' );
							echo '</p></div>';
						}
					}
					else {
						echo '<div class="updated"><p>';
						_e( "Settings have been updated!", 'protect-admin-account' );
						echo '</p></div>';
					}
				}
				else {
					echo '<div class="error"><p>';
					_e( "Settings are not saved.", 'protect-admin-account' );
					echo '</p></div>';
				}
			}
		}
		else {
			echo '<div class="error"><p>';
			_e( "Sorry, nonce verification failed. Settings are not saved.", 'protect-admin-account' );
			echo '</p></div>';
			exit;
		}
	}
}